package com.io.yutian.thewardungeon.data;

import com.io.yutian.thewardungeon.util.ClassUtil;
import com.io.yutian.thewarskyblocklib.util.StringUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LineMetadata {

    private String source;
    private Map<String, String> metadata = new HashMap<>();

    public LineMetadata(String source, Map<String, String> metadata) {
        this.source = source;
        this.metadata = metadata;
    }

    public String getSource() {
        return source;
    }

    public boolean contains(String key) {
        return metadata.containsKey(key);
    }

    public String getString(String key) {
        return getString(key, "");
    }

    public String getString(String key, String defaultValue) {
        String s = key.toLowerCase();
        return metadata.getOrDefault(s, defaultValue);
    }

    public int getInt(@NotNull String key) {
        return getInt(key, 0);
    }

    public int getInt(@NotNull String key, @NotNull int defaultValue) {
        String s = this.metadata.get(key.toLowerCase());
        if (s == null) {
            return defaultValue;
        }
        try {
            return Integer.parseInt(s);
        } catch (Exception e) {
            return defaultValue;
        }
    }

    public boolean getBoolean(@NotNull String key) {
        return getBoolean(key, false);
    }

    public boolean getBoolean(@NotNull String key, @NotNull boolean defaultValue) {
        String s = this.metadata.get(key.toLowerCase());
        if (s == null) {
            return defaultValue;
        }
        try {
            return Boolean.parseBoolean(s);
        } catch (Exception e) {
            return defaultValue;
        }
    }

    public double getDouble(@NotNull String key) {
        return getDouble(key, 0.0D);
    }

    public double getDouble(@NotNull String key, @NotNull double defaultValue) {
        String s = this.metadata.get(key.toLowerCase());
        if (s == null) {
            return defaultValue;
        }
        try {
            return Double.parseDouble(s);
        } catch (Exception e) {
            return defaultValue;
        }
    }

    public float getFloat(@NotNull String key) {
        return getFloat(key, 0.0F);
    }

    public float getFloat(@NotNull String key, @NotNull float defaultValue) {
        String s = this.metadata.get(key.toLowerCase());
        if (s == null) {
            return defaultValue;
        }
        try {
            return Float.parseFloat(s);
        } catch (Exception e) {
            return defaultValue;
        }
    }

    public long getLong(@NotNull String key) {
        return getLong(key, 0L);
    }

    public long getLong(@NotNull String key, @NotNull long defaultValue) {
        String s = this.metadata.get(key.toLowerCase());
        if (s == null) {
            return defaultValue;
        }
        try {
            return Long.parseLong(s);
        } catch (Exception e) {
            return defaultValue;
        }
    }

    public <T> List<T> getList(@NotNull String key, @NotNull Class<T> clazz) {
        return getList(key, clazz, new ArrayList<>());
    }

    public <T> List<T> getList(@NotNull String key, @NotNull Class<T> clazz, @NotNull List<T> defaultValue) {
        String s = this.metadata.get(key.toLowerCase());
        if (s == null) {
            return defaultValue;
        }
        if (!s.startsWith("[") || !s.endsWith("]")) {
            return defaultValue;
        }
        try {
            String s1 = s.substring(1, s.length()-1);
            List<T> list = new ArrayList<>();
            if (s1.contains(",")) {
                if (s1.startsWith(",") || s1.endsWith(",")) {
                    return defaultValue;
                }
                String[] array = s1.split(",");
                for (String arg : array) {
                    T obj = loadValue(arg, clazz);
                    if (obj != null) {
                        list.add(obj);
                    }
                }
            } else {
                T obj = loadValue(s1, clazz);
                if (obj != null) {
                    list.add(obj);
                }
            }
            return list;
        } catch (Exception e) {
            e.printStackTrace();
            return defaultValue;
        }
    }

    private static <T> @Nullable T loadValue(String arg, Class<T> clazz) {
        Class<?> clazz1 = ClassUtil.PRIMITIVE_TO_WRAPPER.getOrDefault(clazz, clazz);
        if (clazz1.equals(String.class)) {
            if (arg.startsWith("\"") && arg.endsWith("\"")) {
                String arg1 = arg.substring(1, arg.length()-1);
                return (T) arg1;
            }
            return (T) arg;
        } else if (clazz1.equals(Integer.class)) {
            if (!StringUtil.isInt(arg)) {
                return null;
            } else {
                return (T)(Integer)Integer.parseInt(arg);
            }
        } else if (clazz1.equals(Double.class)) {
            if (!StringUtil.isDouble(arg)) {
                return null;
            } else {
                return (T)(Double)Double.parseDouble(arg);
            }
        } else if (clazz1.equals(Float.class)) {
            if (!StringUtil.isFloat(arg)) {
                return null;
            } else {
                return (T)(Float)Float.parseFloat(arg);
            }
        } else if (clazz1.equals(Long.class)) {
            if (!StringUtil.isLong(arg)) {
                return null;
            } else {
                return (T)(Long)Long.parseLong(arg);
            }
        } else if (clazz1.equals(Byte.class)) {
            if (!StringUtil.isByte(arg)) {
                return null;
            } else {
                return (T)(Byte)Byte.parseByte(arg);
            }
        } else if (clazz1.equals(Boolean.class)) {
            if (!StringUtil.isBoolean(arg)) {
                return null;
            } else {
                return (T)(Boolean)Boolean.parseBoolean(arg);
            }
        }
        return null;
    }

    @Override
    public String toString() {
        return "LineMetadata{" + source + "}";
    }

}
